import 'dart:convert';
import 'dart:typed_data';
import 'package:encrypt/encrypt.dart' as encrypt;
import 'package:fast_rsa/fast_rsa.dart';
import 'package:flutter/cupertino.dart';
import 'package:m2p_flutter_base/m2p_encrypt_decrypt_config.dart';

class M2PEncryptAndDecrypt {
  static final encrypt.IV iv = encrypt.IV.fromLength(0);

  static String toEncryptAES({required String rawText, required String key}) {
    final encrypt.Encrypter encrypter = encrypt.Encrypter(
        encrypt.AES(encrypt.Key.fromUtf8(key), mode: encrypt.AESMode.ecb));
    return encrypter.encrypt(rawText, iv: iv).base64;
  }

  static String toDecryptAES(
      {required String encryptedText, required String aesKey}) {
    final Uint8List encryptedBytesWithSalt = base64.decode(encryptedText);

    final Uint8List encryptedBytes =
        encryptedBytesWithSalt.sublist(0, encryptedBytesWithSalt.length);

    final encrypt.Encrypter encrypter = encrypt.Encrypter(
        encrypt.AES(encrypt.Key.fromUtf8(aesKey), mode: encrypt.AESMode.ecb));

    final String decrypted =
        encrypter.decrypt64(base64.encode(encryptedBytes), iv: iv);

    return decrypted;
  }

  static Future<String> toEncryptRSA({required String rawText}) async {
    String rsaEncryptedString = '';
    try {
      rsaEncryptedString =
          await RSA.encryptPKCS1v15(rawText, M2PKeyesConfiguration.rsaPublic!);
      debugPrint(rsaEncryptedString);
    } catch (e) {
      debugPrint('encryptRSA--> Exception');
      rethrow;
    }

    return rsaEncryptedString;
  }

  static Future<String> toDecryptRSA({required String encryptedText}) async {
    String rsaEncryptedString = '';
    try {
      rsaEncryptedString = await RSA.decryptPKCS1v15(
          encryptedText, M2PKeyesConfiguration.rsaPrivate!);
    } catch (e) {
      debugPrint('decryptRSA--> Exception');
      rethrow;
    }
    return rsaEncryptedString;
  }

  static Future<Map<String, dynamic>> requestEncryption(
      {required Map<String, dynamic> request}) async {
    final String rsaEncryptedDeviceId =
        await toEncryptRSA(rawText: M2PKeyesConfiguration.deviceId!);
    final String payload = toEncryptAES(
        rawText: jsonEncode(request), key: M2PKeyesConfiguration.deviceId!);
    return {'encryptedPayload': payload, 'encryptedKey': rsaEncryptedDeviceId};
  }

  static Future<Map<String, dynamic>> getEncryptModel(
      {required String encryptedKey, required String encryptedPayload}) async {
    final String aesKey =
        await M2PEncryptAndDecrypt.toDecryptRSA(encryptedText: encryptedKey);
    final String aesDecryptedString = M2PEncryptAndDecrypt.toDecryptAES(
        encryptedText: encryptedPayload, aesKey: aesKey);
    return json.decode(aesDecryptedString) as Map<String, dynamic>;
  }
}
