import 'package:flutter/material.dart';

class CustomButton extends StatefulWidget {
  const CustomButton(
    this.text, {
    Key? key,
    this.fontWeight = FontWeight.w600,
    this.leadingWidget = const Expanded(
      child: SizedBox.shrink(),
    ),
    this.trailingWidget = const Expanded(
      child: SizedBox.shrink(),
    ),
    this.buttonBackgroundColor = const Color(0xffEA6D48),
    this.borderColor = const Color(0xffEA6D48),
    this.textAlign = TextAlign.left,
    this.onTap,
    this.textColor = Colors.white,
    this.fontSize = 18,
    this.isUnderLine = false,
    this.isLeading = false,
    this.isTrailing = false,
    this.isSingleLine = false,
    this.trailingIconData = Icons.settings,
    this.cardElevation = 0.0,
    this.cardBorderRadius = 8.0,
    this.axis = Axis.horizontal,
    this.alignment = MainAxisAlignment.center,
    this.maxLines,
    this.padding = 5.0,
    this.isEnabled = true,
    this.isRemoveExtraPadding = false,
    this.width,
  }) : super(key: key);
  final String? text;
  final double cardElevation;
  final double cardBorderRadius;
  final Color buttonBackgroundColor;
  final Color borderColor;
  final TextAlign textAlign;
  final GestureTapCallback? onTap;
  final bool isUnderLine;
  final bool isSingleLine;
  final bool isLeading;
  final bool isTrailing;
  final int? maxLines;
  final Widget leadingWidget;
  final Widget trailingWidget;
  final FontWeight? fontWeight;
  final Color textColor;
  final IconData? trailingIconData;
  final Axis axis;
  final MainAxisAlignment alignment;
  final double fontSize;
  final bool isEnabled;
  final double padding;
  final bool isRemoveExtraPadding;
  final double? width;

  @override
  // ignore: library_private_types_in_public_api
  _CustomButtonState createState() => _CustomButtonState();
}

class _CustomButtonState extends State<CustomButton> {
  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        if (widget.onTap != null && widget.isEnabled) {
          widget.onTap!();
        }
      },
      child: SizedBox(
        width: MediaQuery.of(context).size.width,
        height: 56,
        child: MediaQuery.removePadding(
          context: context,
          removeBottom: widget.isRemoveExtraPadding,
          removeLeft: widget.isRemoveExtraPadding,
          removeRight: widget.isRemoveExtraPadding,
          removeTop: widget.isRemoveExtraPadding,
          child: Card(
            shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(widget.cardBorderRadius),
                side: BorderSide(
                    width: 0.5,
                    color: widget.borderColor
                        .withOpacity(widget.isEnabled ? 1 : 0.3))),
            color: widget.buttonBackgroundColor
                .withOpacity(widget.isEnabled ? 1 : 0.3),
            elevation: widget.cardElevation,
            child: Padding(
              padding: const EdgeInsets.all(0.0),
              child: Flex(
                direction: widget.axis,
                mainAxisAlignment: widget.alignment,
                children: <Widget>[
                  if (widget.isTrailing) widget.trailingWidget,
                  if (widget.text != null)
                    Padding(
                      padding: EdgeInsets.all(widget.padding),
                      child: Text(
                        widget.text.toString(),
                        textAlign: widget.textAlign,
                        style: Theme.of(context).textTheme.button!.copyWith(
                                // fontSize: widget.fontSize,
                                color: widget.textColor.withOpacity(
                              widget.isEnabled ? 1 : 0.3,
                            )),
                        maxLines: widget.maxLines,
                        overflow:
                            widget.isSingleLine ? TextOverflow.ellipsis : null,
                      ),
                    ),
                  if (widget.isLeading) widget.leadingWidget
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }

  @override
  void initState() {
    super.initState();
  }
}
